import argparse
import fnmatch, re
import sys
from .util import common_parser, common_args, catch_sigint
from .RotateCerts import RotateCerts

def elb_rotate_certs():
    exitcode = 0
    catch_sigint()
    parser = common_parser('ELB Rotate Certificates')

    parser.add_argument(
            '--old',
            help='old server-certificate-name',
            required=True
            )
    parser.add_argument(
            '--new',
            help='new server-certificate-name',
            required=True
            )
    parser.add_argument(
            '--regex',
            help='use regex instead of simple match',
            action='store_true'
            )
    parser.add_argument(
            'elb',
            metavar='ELB',
            nargs='+',
            type=str,
            help='list of ELBs or regexes to match'
            )

    args = parser.parse_args()
    common_args(args)

    rc = RotateCerts(args.region)
    cert_arn_old = u'arn:aws:iam::%s:server-certificate/%s' % (
            rc.get_account_id(),
            args.old
            )
    cert_arn_new = u'arn:aws:iam::%s:server-certificate/%s' % (
            rc.get_account_id(),
            args.new
            )
    elbs = rc.get_all_elbs()

    for elb in elbs:
        # Match ELB names
        matches = 0
        for r in args.elb:
            if args.regex:
                match = re.match(r, elb.name)
            else:
                match = fnmatch.fnmatch(elb.name, r)
            if match:
                matches += 1;

        if matches == 0:
            continue

        for l in elb.listeners:
            if l.ssl_certificate_id == cert_arn_old:
                sys.stdout.write('update elb %s port %d\n' % (
                    elb.name,
                    l.load_balancer_port
                    ))
                ret = rc.update_elb(
                        elb.name,
                        l.load_balancer_port,
                        cert_arn_new
                        )
                if ret == False:
                    exitcode = 1

    sys.exit(exitcode)



import argparse

def common_parser(description='untitled'):
    parser = argparse.ArgumentParser(
        description=description,
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)

    parser.add_argument(
            '-a',
            '--auto',
            help='auto-detect',
            action='store_true'
            )

    parser.add_argument(
            '-r',
            '--region',
            help='ec2 region'
            )

    return parser

import boto.utils
def common_args(args):

    # Region setting:
    # 1. Prefer command-line --region
    # 2. Use instance metadata when --auto
    # 3. Default to us-east-1
    local_region = None
    if args.auto:
        identity = boto.utils.get_instance_identity(timeout=1, num_retries=5)
        try:
            local_region = identity['document']['region']
        except KeyError:
            pass
    if not args.region:
        args.region = local_region
    else:
        local_region = args.region
    if not args.region:
        args.region = 'us-east-1'

import signal
import sys
def cli_signal_handler(signal, frame):
    sys.exit(1)
def catch_sigint():
    signal.signal(signal.SIGINT, cli_signal_handler)



import boto
import boto.ec2.elb
import boto.utils
from boto.exception import BotoServerError
import sys
import time

class RotateCerts():
    """
    ELB Certificate Rotation
    """

    def __init__(self, region):
        self.region = region
        self.rate_limit_delay = 0
        self.elb = boto.ec2.elb.connect_to_region(self.region)
        self.account_id = None

    def get_account_id(self):
        if not self.account_id:
            iam = boto.connect_iam()
            self.account_id = iam.get_user()['get_user_response']['get_user_result']['user']['arn'].split(':')[4]
        return self.account_id

    def get_all_elbs(self):
        self.elbs = list()
        marker = None

        while True:
            elbs = self.wrap_aws_call(
                    self.elb.get_all_load_balancers,
                    marker=marker
                    )
            self.elbs.extend(elbs)
            if elbs.next_marker:
                marker = elbs.next_marker
            else:
                break

        return self.elbs

    def update_elb(self, name, port, new):
        try:
            ret = self.wrap_aws_call(
                    self.elb.set_lb_listener_SSL_certificate,
                    name,
                    port,
                    new
                    )
        except BotoServerError as e:
            sys.stderr.write('failed updating elb %s port %s: %s\n' % (
                name, port, e))
            return False
        return ret

    def wrap_aws_call(self, awsfunc, *args, **nargs):
        """
        Wrap AWS call with Rate-Limiting backoff
        Gratefully taken Netflix/security_monkey
        """
        attempts = 0

        while True:
            attempts = attempts + 1
            try:
                if self.rate_limit_delay > 0:
                    time.sleep(self.rate_limit_delay)

                retval = awsfunc(*args, **nargs)

                if self.rate_limit_delay > 0:
                    self.rate_limit_delay = self.rate_limit_delay / 2

                return retval

            except BotoServerError as e:
                if e.error_code == 'Throttling':
                    if self.rate_limit_delay == 0:
                        self.rate_limit_delay = 1
                        sys.stderr.write('rate-limited: attempt %d\n' % \
                                attempts)
                    elif self.rate_limit_delay < 16:
                        self.rate_limit_delay = self.rate_limit_delay * 2
                        sys.stderr.write('rate-limited: attempt %d\n' % \
                                attempts)
                    else:
                        raise e
                else:
                    raise e



import os
import re
from elb_rotate_certs import __version__
from setuptools import setup, find_packages

setup(
    name='elb-rotate-certs',
    version=__version__,
    author="Chris Maxwell",
    author_email="chris@wrathofchris.com",
    description="ELB Certificate Rotation Tool",
    url = "https://github.com/WrathOfChris/elb-rotate-certs",
    download_url = 'https://github.com/WrathOfChris/elb-rotate-certs/tarball/%s' % __version__,
    license="Apache",
    include_package_data=True,
    packages=find_packages(),
    install_requires=[
        'boto'
    ],
    entry_points={
        "console_scripts": [
            "elb-rotate-certs = elb_rotate_certs.cli:elb_rotate_certs"
        ]
    }
)
